/*
    LLamy
    Copyright (C) 2004 Andrej Krutak

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "func.h"
#include "playlevel.h"

#include <stdio.h>
#include <string.h>
#include <stdarg.h>

//I guess that the algorithms in this file are far too simple to be described... :-)

extern SDL_Surface *screen;
extern int flip, sw, sh;
static Mix_Chunk *waves[SOUND_NOITEMS];
static int stopfsimage;

char *my_strcat(char *buf, char *str1, char *str2)
{
	strcpy(buf, str1);
	strcat(buf, str2);
	return buf;
}

char *stripline(char *line)
{
	if (line[strlen(line)-1]=='\n')
		line[strlen(line)-1]='\0';
	return line;
}

void drawImage(SDL_Surface *target, SDL_Surface *picture, int x, int y, int w, int h, int x1, int y1)
{
	SDL_Rect src, dst;

	src.x = x;
	src.y = y;
	src.w = w;
	src.h = h;
	
	dst = src;
	dst.x = x1;
	dst.y = y1;

	if ( SDL_BlitSurface(picture, &src, target, &dst) < 0 ) {
		fprintf(stderr, "Blit failed: %s\n", SDL_GetError());
	}
}

void drawWholeImage(SDL_Surface *picture)
{
	drawImage(screen, picture, 0, 0, picture->w, picture->h, 0, 0);
}

void updateScreen(void)
{
	SDL_Rect update;

	update.x=0;
	update.y=0;
	update.w=screen->w;
	update.h=screen->h;

	if ( flip ) SDL_Flip(screen);
	else SDL_UpdateRects(screen, 1, &update);
}

SDL_Surface *myLoadImage(char *buf, int show, int discard)
{
	SDL_Surface *picture;

	picture = IMG_Load(buf);
	if (picture == NULL) {
		fprintf(stderr, "Couldn't load: %s\n", SDL_GetError());
		return 0;
	}

	if (show) {
		drawWholeImage(picture);
		updateScreen();
	}
	
	if (discard)
		SDL_FreeSurface(picture);

	if (!discard)
		return picture;
	return 0;
}

Uint32 stopImageFullscreen(Uint32 interval, void *param)
{
	stopfsimage=1;
	return 0;
}

void showImageFullscreen(SDL_Surface *im, int time, char *path)
{
	SDL_Event event;
	int done = 0;

	stopfsimage=0;
	SDL_Surface *i;
	if (im==0) {
		i=myLoadImage(path, 1, 1);
		SDL_FreeSurface(i);
	}  else {
		drawWholeImage(im);
		updateScreen();
	}
	SDL_TimerID t=SDL_AddTimer(time, stopImageFullscreen, 0);

	while ( !done && !stopfsimage) {
		while ( SDL_PollEvent(&event) ) {
			switch (event.type) {
				case SDL_KEYDOWN:
				case SDL_MOUSEBUTTONDOWN:
					done=1;
					break;
			}
		}
	}
	SDL_RemoveTimer(t);
}

int isInRect(SDL_Rect r, int posx, int posy, int sx, int sy, int x, int y)
{
	if (r.x+r.w<posx || r.y+r.h<posy)
		return 0;
	if (r.x>posx+sx || r.y>posy+sy)
		return 0;

	r.x-=posx;
	r.y-=posy;

	if (x>=r.x && x<r.x+r.w && y>=r.y && y<r.y+r.h)
		return 1;
	return 0;
}

void drawItemAtRect(SDL_Surface *surface, int x, int y, SDL_Rect r, int posx, int posy, int sx, int sy)
{
	if (r.x+r.w<posx || r.y+r.h<posy)
		return;
	if (r.x>posx+sx || r.y>posy+sy)
		return;

	drawImage(screen, surface, x, y, r.w, r.h, r.x-posx, r.y-posy);
}

void drawText(SDL_Surface *target, int x, int y, SDL_Surface *font, char *fmt, ...)
{
	int i, l;
	int chw=font->w/256;
	char		buf[2048];
	va_list     va;
	int cy=0, cx=0;

    va_start(va, fmt);
    vsprintf(buf, fmt, va);
    va_end(va);

	l=strlen(buf);

	for (i=0; i<l; i++) {
		if (buf[i]=='\n') {
			cy+=15;
			cx=0;
		} else if (buf[i]=='\t') {
			cx+=TAB_WIDTH*chw;
			cx=(cx/(TAB_WIDTH*chw))*(TAB_WIDTH*chw);
		} else {
			drawImage(target, font, buf[i]*chw, 0, chw, font->h, x+cx, y+cy);
			cx+=chw;
		}
	}
}

int giveQuestion(char *path, SDL_Rect ryes, SDL_Rect rno)
{
	SDL_Rect dest, src;

	SDL_Surface *i_quit=IMG_Load(path);
	SDL_SetAlpha(i_quit, SDL_SRCALPHA, 180);
	src.y=src.x=0;
	src.w=i_quit->w;
	src.h=i_quit->h;
	
	dest.x = (screen->w-src.w)/2;
	dest.y = (screen->h-src.h)/2;
	dest.w = src.w;
	dest.h = src.h;

	SDL_BlitSurface(i_quit, &src, screen, &dest);

	SDL_FreeSurface(i_quit);

	updateScreen();
	SDL_Event event;

	while (1) {
		while ( SDL_PollEvent(&event) ) {
			switch (event.type) {
			case SDL_KEYDOWN:
				if ((event.key.keysym.sym==SDLK_ESCAPE) || (event.key.keysym.sym=='n'))
					return 0;
				if ((event.key.keysym.sym==SDLK_RETURN) || (event.key.keysym.sym=='y'))
					return 1;
			case SDL_MOUSEBUTTONDOWN:
				if (event.button.button==SDL_BUTTON_LEFT) {
					if ((event.button.x>(dest.x+ryes.x)) && (event.button.x<(dest.x+ryes.x+ryes.w)) &&
						(event.button.y>(dest.y+ryes.y)) && (event.button.y<(dest.y+ryes.y+ryes.h)))
						return 1;
					else if ((event.button.x>(dest.x+rno.x)) && (event.button.x<(dest.x+rno.x+rno.w)) &&
						(event.button.y>(dest.y+rno.y)) && (event.button.y<(dest.y+rno.y+rno.h)))
						return 0;
				}
			}
		}
	}

	return 1;
}

int my_playSound(int id, int x, int y)
{
	int px=screenXPos();
	int py=screenYPos();
	int mx=mapXSize();
	int my=mapYSize();

	int nx=px+sw/2;
	int ny=py+sw/2;

	int d=255*sqrt((x-nx)*(x-nx)+(y-ny)*(y-ny))/sqrt(mx*mx+my*my);
	int a;
	if (nx<x)
		a=270;
	else a=90;

	//printf("POS: %d %d\n", a, d);
	Mix_SetPosition(0, a, d);

	if (id<SOUND_NOITEMS && id>=0 && waves[id]) {
		Mix_PlayChannel(0, waves[id], 0);
		return 0;
	}
	return 1;
}

Mix_Music * my_playMusicStart(char *path)
{
	if (Mix_PlayingMusic())
		Mix_HaltMusic();

	Mix_Music *x=Mix_LoadMUS(path);
	if (x) {
		Mix_PlayMusic(x, 0);
		Mix_VolumeMusic(32);
	}
	return x;
}

int my_playMusicStop(Mix_Music *x)
{
	if (Mix_PlayingMusic())
		Mix_HaltMusic();
	Mix_FreeMusic(x);
	return 0;
}

int my_loadSounds()
{
	int i;
	char buf[100];

	for (i=0; i<SOUND_NOITEMS; i++) {
		sprintf(buf, "sound/%02d.wav", i);
		waves[i]=Mix_LoadWAV(buf);
		if (!waves[i])
			printf("! %s\n", buf);
	}
	return 0;
}

int my_freeSounds()
{
	for (int i=0; i<SOUND_NOITEMS; i++)
		if (waves[i]) Mix_FreeChunk(waves[i]);
	return 0;
}