/*
    LLamy
    Copyright (C) 2004 Andrej Krutak

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <sdl.h>
#include <sdl_image.h>
#include <sdl_mixer.h>
#include <sge.h>

#ifndef _FUNC_H
#define _FUNC_H

enum {
//SOUNDS
	SOUND_KILLED=0,
	SOUND_EXPLODE,
	SOUND_YIPEE,
	SOUND_DIG,
	SOUND_STAIR,

	SOUND_NOITEMS,

//MUSIC
	MUSIC_NOITEMS=2, //maximal count of songs to play

//OTEHR
	TAB_WIDTH=4,
};

//--------misc. functions----------

//strcat=> 'buf=str1+str2'; returns 'buf'
char *my_strcat(char *buf, char *str1, char *str2);
//removes eol character from the end of line; returns 'line'
char *stripline(char *line);

//--------image manipulation----------

//draws 'source' to 'target'. x, y, w, h are coordinates in 'source'; x1, y1 are coordinates in 'target'
void drawImage(SDL_Surface *target, SDL_Surface *source, int x, int y, int w, int h, int x1, int y1);

//draws 'picture' on the screen (screen must be initialized, of course...)
void drawWholeImage(SDL_Surface *picture);

//loads image 'buf', if show==1, it will be drawn on the screen (drawWholeImage);
//if discard==1, the image will be discarded immediately; otherwise it will be returned
SDL_Surface *myLoadImage(char *buf, int show, int discard);

//shows the image 'im' on the screen for 'time' milliseconds; if 'im'=0, the image will be loaded from 'path'
void showImageFullscreen(SDL_Surface *im, int time, char *path=0);

//draws formatted text (like printf...) to 'target', position x, y, using font 'font'...
void drawText(SDL_Surface *target, int x, int y, SDL_Surface *font, char *fmt, ...);

//draws 'surface' (the offset in this image is specified by x, y), which should be on the place specified by 'r'
//posx, posy are position of the left-top corned of the screen; sx, sy is the size of the screen
void drawItemAtRect(SDL_Surface *surface, int x, int y, SDL_Rect r, int posx, int posy, int sx, int sy);

//checks whether the rect is in screen; returns 1 if yes, 0 otherwise
int isInRect(SDL_Rect r, int posx, int posy, int sx, int sy, int x, int y);

//redraws screen, by respecting flipping, screen size etc.
void updateScreen(void);

//shows image specified by 'path', centers it and the user has to respond by clicking one of the 2 buttons,
//specified by 'ryes' and 'rno'. Function returns 0 if item specified by 'rno' was clicked; 1 otherwise...
int giveQuestion(char *path, SDL_Rect ryes, SDL_Rect rno);

//loads program sounds
int my_loadSounds();

//frees sounds loaded
int my_freeSounds();

//plays sound 'id' at the position 'x', 'y' in the map...
int my_playSound(int id, int x, int y);

//starts the playback of the sound 'path'; returns the loaded file (useful by freeing by my_playMusicStop)
Mix_Music * my_playMusicStart(char *path);

//stops playback of the sound 'x', previosly started by <my_playMusicStart>
int my_playMusicStop(Mix_Music *x);

#endif
