/*
    LLamy
    Copyright (C) 2004 Andrej Krutak

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "llamy.h"

//draws current settings checkboxes :-)
void drawSettings(SDL_Surface *menu1, SDL_Rect b1, SDL_Rect b3, SDL_Rect f, int cb, int cf)
{
	Uint32 clr;
	int i, j;

	drawWholeImage(menu1);
	clr=SDL_MapRGB(screen->format, 0, 0, 0);

	if (cf) {
		for (i=0; i<23; i++)
			for (j=0; j<23; j++)
				_PutPixel(screen, f.x+i, f.y+j, clr);
	}
	if (cb==16) {
		for (i=0; i<23; i++)
			for (j=0; j<23; j++)
				_PutPixel(screen, b1.x+i, b1.y+j, clr);
	}
	if (cb==32) {
		for (i=0; i<23; i++)
			for (j=0; j<23; j++)
				_PutPixel(screen, b3.x+i, b3.y+j, clr);
	}
	updateScreen();
}

//shows settings 'dialog'
int showSettings(char *path)
{
	SDL_Surface *menu1=myLoadImage(path, 0, 0);
	SDL_Rect b1={31, 86, 155, 24};
	SDL_Rect b3={31, 114, 155, 24};
	SDL_Rect f={31, 173, 173, 24};
	SDL_Rect ok={0, 433, 67, 37};
	SDL_Rect cancel={108, 433, 88, 37};

	int state=1;
	SDL_Event event;
	int done = 0;

	int cb=lconfig.sb;
	int cf=lconfig.fullscreen;

	drawSettings(menu1, b1, b3, f, cb, cf);

	while ( !done ) {
		while ( SDL_PollEvent(&event) ) {
			switch (event.type) {
				case SDL_KEYDOWN:
					if (event.key.keysym.sym==SDLK_ESCAPE)
						done=-1;
					else if (event.key.keysym.sym==SDLK_RETURN)
						done=1;
					break;
				case SDL_QUIT:
					done = 0;
					break;
				case SDL_MOUSEBUTTONDOWN:
					if (INRECT(event.button.x, event.button.y, b1.x, b1.y, b1.w, b1.h)) {
						cb=16;
						drawSettings(menu1, b1, b3, f, cb, cf);
					} else if (INRECT(event.button.x, event.button.y, b3.x, b3.y, b3.w, b3.h)) {
						cb=32;
						drawSettings(menu1, b1, b3, f, cb, cf);
					} else if (INRECT(event.button.x, event.button.y, f.x, f.y, f.w, f.h)) {
						cf=!cf;
						drawSettings(menu1, b1, b3, f, cb, cf);
					} else if (INRECT(event.button.x, event.button.y, ok.x, ok.y, ok.w, ok.h)) {
						lconfig.sb=cb;
						lconfig.fullscreen=cf;
						done=1;
					} else if (INRECT(event.button.x, event.button.y, cancel.x, cancel.y, cancel.w, cancel.h)) {
						done=-1;
					}
					break;
				default:
					break;
			}
		}
	}

	if (done==1)
		return 1;
	return 0;
}

//drwa available levels etc.
void drawLevelSelect(SDL_Surface *menu1, SDL_Surface *font, deque<leveldef> &l, int cl)
{
	const int PGITEMS=20;
	drawWholeImage(menu1);

	int page=cl/PGITEMS;
	int pages=l.size()/PGITEMS;
	int i, j, iop;

	if (page*PGITEMS>l.size()) {
		iop=page*PGITEMS-l.size();
	} else {
		iop=PGITEMS;
	}
	if (pages==0)
		iop=l.size();

	//drawText(300, 50, font, "%d %d %d %d", pages, iop, l.size(), cl);

	for (i=0; i<iop; i++) {
		if (i+page*PGITEMS>=l.size())
			break;
		drawText(screen, 31, 100+13*i, font, "%c[%d] %s", ((i+page*PGITEMS)>lconfig.maxlevel)?'~':' ', i+page*PGITEMS+1, l[i+page*PGITEMS].name);
	}

	Uint32 clr;
	clr=SDL_MapRGB(screen->format, 255, 255, 255);

	for (i=0; i<350; i++) {
		for (j=0; j<13; j++) {
			_PutPixelAlpha(screen, 30+i, 100+(cl%PGITEMS)*13+j, clr, 70*(350-i)/350);
		}
	}
	updateScreen();
}

//show level selection 'dialog'
int showLevelSelect()
{
	SDL_Surface *menu1=myLoadImage("img/menu4.jpg", 1, 0);
	SDL_Rect b1={19, 56, 35, 42};
	SDL_Rect b2={19, 362, 35, 42};
	SDL_Rect ok={0, 433, 67, 37};
	SDL_Rect cancel={108, 433, 88, 37};
	char buf[100];

	SDL_Surface *font=IMG_Load(my_strcat(buf, "img/", "font.png"));
	if (font==0) {
		SDL_FreeSurface(menu1);
		printf("!%s\n", buf);
		return 1;
	}
	SDL_SetColorKey(font, SDL_SRCCOLORKEY, SDL_MapRGB(font->format, 255, 0, 255));

	int state=1;
	SDL_Event event;
	int done = 0;

	int cl=lconfig.curlevel;

	drawLevelSelect(menu1, font, lf, cl);

	while ( !done ) {
		while ( SDL_PollEvent(&event) ) {
			switch (event.type) {
				case SDL_KEYDOWN:
					if (event.key.keysym.sym==SDLK_ESCAPE)
						done=-1;
					else if (event.key.keysym.sym==SDLK_RETURN) {
						lconfig.curlevel=cl;
						done=1;
					} else if (event.key.keysym.sym==SDLK_UP)
						goto goup;
					else if (event.key.keysym.sym==SDLK_DOWN)
						goto godown;
					break;
				case SDL_QUIT:
					done = 0;
					break;
				case SDL_MOUSEBUTTONDOWN:
					if (INRECT(event.button.x, event.button.y, b1.x, b1.y, b1.w, b1.h)) {
goup:
						if (cl>0)
							cl--;
						drawLevelSelect(menu1, font, lf, cl);
					} else if (INRECT(event.button.x, event.button.y, b2.x, b2.y, b2.w, b2.h)) {
godown:
						if (cl<lconfig.maxlevel)
							cl++;
						drawLevelSelect(menu1, font, lf, cl);
					} else if (INRECT(event.button.x, event.button.y, ok.x, ok.y, ok.w, ok.h)) {
						lconfig.curlevel=cl;
						done=1;
					} else if (INRECT(event.button.x, event.button.y, cancel.x, cancel.y, cancel.w, cancel.h)) {
						done=-1;
					}
					break;
				default:
					break;
			}
		}
	}

	if (done==1)
		return 1;
	return 0;
}

//show normal menu, return selected item
int showMenu2(SDL_Rect *rects, int nrects, char *img1, char *img2)
{
	SDL_Surface *menu1=myLoadImage(img1, 1, 0);
	SDL_Surface *menu2=myLoadImage(img2, 0, 0);

	int state=1;
	SDL_Event event;
	int done = 0;
	int i;
	int f=0, ff;

	while ( !done ) {
		while ( SDL_PollEvent(&event) ) {
			switch (event.type) {
				case SDL_KEYDOWN:
					if (event.key.keysym.sym==SDLK_ESCAPE)
						done=-1;
					if (event.key.keysym.sym=='s')
						done=1;
					if (event.key.keysym.sym=='l')
						done=2;
					if (event.key.keysym.sym=='t')
						done=3;
					if (event.key.keysym.sym=='x' || event.key.keysym.sym=='e')
						done=4;
					break;
				case SDL_QUIT:
					done = -1;
					break;
				case SDL_MOUSEBUTTONDOWN:
					if (event.button.button==SDL_BUTTON_LEFT) {
						for (i=0; i<nrects; i++) {
							if ((event.button.x>=rects[i].x) && (event.button.x<=(rects[i].x+rects[i].w)) &&
								(event.button.y>=rects[i].y) && (event.button.y<=(rects[i].y+rects[i].h))) {
								done=i+1;
								break;
							}
						}
					}
					break;
				case SDL_MOUSEMOTION:
					ff=f;
					f=0;
					for (i=0; i<nrects; i++) {
						if ((event.motion.x>=rects[i].x) && (event.motion.x<=(rects[i].x+rects[i].w)) &&
							(event.motion.y>=rects[i].y) && (event.motion.y<=(rects[i].y+rects[i].h))) {
							if (ff!=(i+1)) {
								drawWholeImage(menu1);
								drawImage(screen, menu2, rects[i].x, rects[i].y, rects[i].w, rects[i].h, rects[i].x, rects[i].y);
								updateScreen();
								//printf("%d", i+1);
								f=i+1;
							} else f=i+1;
							break;
						}
					}

					if (f==0 && ff!=0) {
						//printf("%d", f);
						drawWholeImage(menu1);
						updateScreen();
					}
					break;

				default:
					break;
			}
		}
	}

	return (done>0)?(done-1):done;
}

//main menu loop
int showMenu()
{
	SDL_Rect r[4];
	int rv;

	r[0].x=0;
	r[0].y=134;
	r[0].w=250;
	r[0].h=47;

	r[1].x=0;
	r[1].y=180;
	r[1].w=250;
	r[1].h=44;

	r[2].x=0;
	r[2].y=223;
	r[2].w=250;
	r[2].h=45;

	r[3].x=0;
	r[3].y=267;
	r[3].w=250;
	r[3].h=39;

	//Load level definitions
	nl=0;

	leveldef cld;

	FILE *f=fopen("maps/levels.txt", "r");
	if (!f) {
		printf("!maps/levels.txt\n");
		return 1;
	}
	lf.clear();
	while (!feof(f)) {
		fgets(cld.name, 100, f);
		if (cld.name[strlen(cld.name)-1]=='\n')
			cld.name[strlen(cld.name)-1]='\0';
		fgets(cld.path, 100, f);
		if (cld.path[strlen(cld.path)-1]=='\n')
			cld.path[strlen(cld.path)-1]='\0';
		memcpy(cld.name, &cld.name[2], sizeof(cld.name)-2);

		lf.push_back(cld);
	}
	fclose(f);
	nl=lf.size();

restart:
	if ((rv=showMenu2(r, 4, "img/menu.jpg", "img/menu2.jpg"))==-1)
		return 0;
	else {
		switch (rv) {
		case 0:
			if (playLevel(lf[lconfig.curlevel].path, lf[lconfig.curlevel].name)==-3) {
				if (lconfig.curlevel>=lconfig.maxlevel && lconfig.maxlevel+1<nl) {
					lconfig.maxlevel++;
				}
				if (lconfig.curlevel+1<lconfig.maxlevel)
					lconfig.curlevel++;
			}
			break;
		case 1:
			showLevelSelect();
			break;
		case 2:
			if (showSettings("img/menu3.jpg"))
				return 1;
			break;
		case 3:
			//WONT_BE: confirm?
			return 0;
		}
	}
	goto restart;
}

//save current configuration
void saveConfig()
{
	FILE *f=fopen("llamy.ini", "w");
	if (!f) {
		printf("Can't create config file!!!\n");
	} else {
		fprintf(f, "%d %d %d %d %d", lconfig.maxlevel, lconfig.fullscreen, lconfig.sx, lconfig.sy, lconfig.sb);
		fclose(f);
	}
}

//load configuration
void loadConfig()
{
	FILE *f=fopen("llamy.ini", "r");
	if (!f) {
		printf("Config file doesn't exist - trying to create...\n");
		lconfig.curlevel=0;
		lconfig.maxlevel=0;
		lconfig.fullscreen=0;
		lconfig.sb=16;
		lconfig.sx=640;
		lconfig.sy=480;

		saveConfig();
	} else {
		fscanf(f, "%d %d %d %d %d", &lconfig.maxlevel, &lconfig.fullscreen, &lconfig.sx, &lconfig.sy, &lconfig.sb);
		lconfig.curlevel=lconfig.maxlevel;
		fclose(f);
	}
}

//main loop
int main(int argc, char **argv)
{
	int desired_bpp;
	int video_flags;
	int audio_rate;
	Uint16 audio_format;
	int audio_channels; 
	int audio_init=0;

	flip=0;

	loadConfig();

	//SDL init...
	if ( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO|SDL_INIT_TIMER) < 0 ) {
		fprintf(stderr, "Couldn't initialize SDL: %s\n", SDL_GetError());
		exit(1);
	}

	atexit(SDL_Quit);

	{
		char buf[100];
		printf("Video driver: %s\n", SDL_VideoDriverName(buf, 100));
	}
reset:
	//load command line options
	video_flags=0;

	sw=lconfig.sx;
	sh=lconfig.sy;
	desired_bpp=lconfig.sb;
	if (lconfig.fullscreen)
		video_flags |= SDL_FULLSCREEN;

	srand(time(0));
	while ( argc > 1 ) {
		if ( strcmp(argv[1], "-width") == 0 ) {
			if ( argv[2] && ((sw = atoi(argv[2])) > 0) ) {
				argv += 2;
				argc -= 2;
			} else {
				fprintf(stderr, "The -width option requires an argument\n");
				exit(1);
			}
		} else
		if ( strcmp(argv[1], "-height") == 0 ) {
			if ( argv[2] && ((sh = atoi(argv[2])) > 0) ) {
				argv += 2;
				argc -= 2;
			} else {
				fprintf(stderr, "The -height option requires an argument\n");
				exit(1);
			}
		} else
		if ( strcmp(argv[1], "-bpp") == 0 ) {
			if ( argv[2] ) {
				desired_bpp = atoi(argv[2]);
				argv += 2;
				argc -= 2;
			} else {
				fprintf(stderr, "The -bpp option requires an argument\n");
				exit(1);
			}
		} else if ( strcmp(argv[1], "-warp") == 0 ) {
			video_flags |= SDL_HWPALETTE;
			argv += 1;
			argc -= 1;
		} else if ( strcmp(argv[1], "-hw") == 0 ) {
			video_flags |= SDL_HWSURFACE;
			argv += 1;
			argc -= 1;
		} else if ( strcmp(argv[1], "-flip") == 0 ) {
			video_flags |= SDL_DOUBLEBUF;
			argv += 1;
			argc -= 1;
		} else if ( strcmp(argv[1], "-fullscreen") == 0 ) {
			video_flags |= SDL_FULLSCREEN;
			argv += 1;
			argc -= 1;
		} else break;
	}

	//initialize audio
	audio_rate = MIX_DEFAULT_FREQUENCY;
	audio_format = MIX_DEFAULT_FORMAT;
	audio_channels = 2; 

	if (!audio_init) {
		audio_init=1;
		if (Mix_OpenAudio(audio_rate, audio_format, audio_channels, 4096) < 0) {
			fprintf(stderr, "Couldn't open audio: %s\n", SDL_GetError());
			return(2);
		} else {
			Mix_QuerySpec(&audio_rate, &audio_format, &audio_channels);
			printf("Opened audio at %d Hz %d bit %s\n", audio_rate, (audio_format&0xFF), (audio_channels > 1) ? "stereo" : "mono");
		}
		my_loadSounds();
	}

	//set video mode etc.
	SDL_WM_SetCaption("LLamy", NULL);

	screen = SDL_SetVideoMode(sw, sh, desired_bpp, video_flags);
	if ( screen == NULL ) {
		fprintf(stderr, "Couldn't set %dx%dx%d video mode: %s\n", sw, sh, desired_bpp, SDL_GetError());
		exit(1);
	}
	printf("Set%s %dx%dx%d mode\n", screen->flags & SDL_FULLSCREEN ? " fullscreen" : "", screen->w, screen->h, screen->format->BitsPerPixel);
	printf("(video surface located in %s memory)\n", (screen->flags&SDL_HWSURFACE) ? "video" : "system");
	if ( screen->flags & SDL_DOUBLEBUF ) {
		printf("Double-buffering enabled\n");
		flip = 1;
	}

	//show menu
	Mix_Music *m=my_playMusicStart("sound/intro.ogg");
	showImageFullscreen(0, 5000, "img/about.jpg");
	my_playMusicStop(m);

	if (showMenu()) { //happens when screen settings change
		goto reset;
	}

	//cleanup
	SDL_Quit();

	lconfig.sx=sw;
	lconfig.sy=sh;
	lconfig.sb=desired_bpp;
	if (video_flags & SDL_FULLSCREEN)
		lconfig.fullscreen=1;
	else lconfig.fullscreen=0;
	
	saveConfig();

	my_freeSounds();
	return 0;
}
