/*
    LLamy
    Copyright (C) 2004 Andrej Krutak

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "map.h"
#include "func.h"
#include <stdio.h>
#include <string.h>

int inline sign(int x)
{
	if (x>0)
		return 1;
	else if (x<0)
		return -1;
	return 0;
}

int inline myabs(int i)
{
	if (i>=0)
		return i;
	return -i;
}

int getNextParam(FILE *f, char *str1, char *str2)
{
	char buf[1000];
	if (feof(f))
		return 0;

skipline:
	if (!fgets(buf, 1000, f))
		return 0;

	if (buf[0]!=':')
		goto skipline;
	strcpy(str1, stripline(&buf[2]));

	if (!fgets(buf, 1000, f))
		return 0;
	strcpy(str2, stripline(buf));
	
	return 1;
}

Map::Map(char *path):loaded(0)
{
	char buf[1000], buf2[1000];
	char texturepath[1000];
	int i;

	//zero all
	printf("\tMap()...");

	strcpy(texturepath, "");

	time=0;

	map=0;
	bk=0;
	solid=0;

	llamyHome=0;
	llamyStart=0;
	llamyEnd=0;

	speedMin=10;
	speedMax=10;
	speedStart=10;
	
	for (i=0; i<MAP_NO_ITEMS; i++) {
		items[i]=0;
	}

	for (i=0; i<MAX_LLAMAS; i++) {
		actions[i].what=-1;
	}

	action_count=0;
	llama_count=0;

	//load settings from file
	FILE *f=fopen(my_strcat(buf, "maps/", path), "r");
	if (!f) {
		printf("!FILE\n");
		return;
	}

	while (getNextParam(f, buf, buf2)) {
		if (!stricmp(buf, "map")) {
			map=IMG_Load(my_strcat(buf, "maps/", buf2));
			if (!map) {
				printf("!%s\n", buf);
				continue;
			}
			printf("Map:%dx%d; ", map->w, map->h);
		} else if (!stricmp(buf, "texture")) {
			int i, j;
			SDL_Surface *texture;
			texture=IMG_Load(my_strcat(buf, "maps/", buf2));
			if (!texture) {
				printf("!%s\n", buf);
				continue;
			}
			printf("tex");

			SDL_LockSurface(map);
			SDL_LockSurface(texture);

			Uint8 *pixels=(Uint8 *)map->pixels;
			Uint8 *tpixels=(Uint8 *)texture->pixels;

			for (i=0; i<map->w; i++) {
				for (j=0; j<map->h; j++) {
					//_PutPixel(llama_motion2[k], llama_motion[k]->w-i-1, j, sge_GetPixel(llama_motion[k], i, j));
					SDL_Color c=sge_GetRGB(map, sge_GetPixel(map, i, j));
					if ((c.r==grColor.r) &&
					(c.g==grColor.g) &&
					(c.b==grColor.b)) {
						_PutPixel(map, i, j, sge_GetPixel(texture, i%texture->w, j%texture->h));
						//pixels[i*3+j*map->w*3+0]=tpixels[(i%texture->w)*3+(j%texture->h)*texture->w*3+0];
						//pixels[i*3+j*map->w*3+1]=tpixels[(i%texture->w)*3+(j%texture->h)*texture->w*3+1];
						//pixels[i*3+j*map->w*3+2]=tpixels[(i%texture->w)*3+(j%texture->h)*texture->w*3+2];
					}
				}
			}

			SDL_UnlockSurface(texture);
			SDL_UnlockSurface(map);

			SDL_FreeSurface(texture);
			printf("ture; ");
		} else if (!stricmp(buf, "bkcolor")) {
			sscanf(buf2, "%d %d %d", &bkColor.r, &bkColor.g, &bkColor.b);
			if (map)
				SDL_SetColorKey(map, SDL_SRCCOLORKEY, SDL_MapRGB(map->format, bkColor.r, bkColor.g, bkColor.b));
		} else if (!stricmp(buf, "bkimage")) {
			bk=IMG_Load(my_strcat(buf, "maps/", buf2));
			if (!bk)
				printf("!%s\n", buf);
		} else if (!stricmp(buf, "solid")) {
			solid=IMG_Load(my_strcat(buf, "maps/", buf2));
			if (!solid)
				printf("!%s\n", buf);
		} else if (!stricmp(buf, "groundcolor")) {
			sscanf(buf2, "%d %d %d", &grColor.r, &grColor.g, &grColor.b);
		} else if (!stricmp(buf, "start")) {
			sscanf(buf2, "%d %d", &start.x, &start.y);
			start.w=40;
			start.h=40;
		} else if (!stricmp(buf, "exit")) {
			sscanf(buf2, "%d %d", &exit.x, &exit.y);
			exit.w=40;
			exit.h=40;
		} else if (!stricmp(buf, "startllamy")) {
			sscanf(buf2, "%d", &llamyStart);
		} else if (!stricmp(buf, "endllamy")) {
			sscanf(buf2, "%d", &llamyEnd);
		} else if (!stricmp(buf, "minspeed")) {
			sscanf(buf2, "%d", &speedMin);
		} else if (!stricmp(buf, "maxspeed")) {
			sscanf(buf2, "%d", &speedMax);
		} else if (!stricmp(buf, "startspeed")) {
			sscanf(buf2, "%d", &speedStart);
		} else if (!stricmp(buf, "i_walk")) {
			sscanf(buf2, "%d", &items[ITEM_WALK]);	
		} else if (!stricmp(buf, "i_dig")) {
			sscanf(buf2, "%d", &items[ITEM_DIG]);	
		} else if (!stricmp(buf, "i_explode")) {
			sscanf(buf2, "%d", &items[ITEM_EXPLODE]);
		} else if (!stricmp(buf, "i_stairs")) {
			sscanf(buf2, "%d", &items[ITEM_STAIRS]);
		} else if (!stricmp(buf, "i_digger")) {
			sscanf(buf2, "%d", &items[ITEM_DIGGER]);
		} else if (!stricmp(buf, "i_block")) {
			sscanf(buf2, "%d", &items[ITEM_BLOCK]);
		} else if (!stricmp(buf, "i_climb")) {
			sscanf(buf2, "%d", &items[ITEM_CLIMB]);
		} else if (!stricmp(buf, "i_wings")) {
			sscanf(buf2, "%d", &items[ITEM_WINGS]);
		} else if (!stricmp(buf, "i_atombomb")) {
			sscanf(buf2, "%d", &items[ITEM_ATOMBOMB]);
		} else {
			printf("\n!?%s\n", buf);
		}
	}

	llamasToStart=llamyStart;
	llamasLastStart=0;
	llamaStartSpeed=speedStart;

	fclose(f);
	printf("OK\n");
	loaded=1;
}

Map::~Map()
{
	printf("\t~Map()...");

	if (map)
		SDL_FreeSurface(map);
	if (bk)
		SDL_FreeSurface(bk);
	if (solid)
		SDL_FreeSurface(solid);

	printf("OK\n");
}

int inline Map::mapIsGround(int x, int y)
{
	/*
	Uint8 *pixels=(Uint8 *)map->pixels;

	if (x>=map->w || x<0 || y>=map->h || y<0)
		return 0;
	if ((pixels[x*3+y*map->w*3+0]==bkColor.r) && (pixels[x*3+y*map->w*3+1]==bkColor.g) && (pixels[x*3+y*map->w*3+2]==bkColor.b))
		return 0;
	else return 1;
	*/
	if (x>=map->w || x<0 || y>=map->h || y<0)
		return 0;

	SDL_Color c=sge_GetRGB(map, sge_GetPixel(map, x, y));
	if ((c.r==bkColor.r) && (c.g==bkColor.g) && (c.b==bkColor.b))
		return 0;
	else return 1;
}

int inline Map::mapSetGround(int x, int y, int erase)
{
	SDL_Color color;
	if (erase)
		color=bkColor;
	else {
		color.r=color.g=color.b=0;
	}

	if (x<0 || x>=map->w || y<0 || y>=map->h)
		return 1;
	if (solid) {
		if (((Uint8 *)solid->pixels)[x*3+y*map->w*3+0]==0)
			return 1;
	}
	Uint8 *pixels=&((Uint8 *)map->pixels)[x*3+y*map->w*3+0];

	*pixels++=color.r;
	*pixels++=color.g;
	*pixels++=color.b;
	return 0;
}

int Map::mapIsBlocked(int x, int y)
{
	int i;

	for (i=0; i<llama_count; i++) {
		if (llamas[i].isAlive() && llamas[i].getAction()==ITEM_BLOCK) { //only process, if he's alive :)
			if (INRECT(x, y, llamas[i].getX(), llamas[i].getY(), LLAMA_SIZE, LLAMA_SIZE))
				return 1;
		}
	}
	return 0;
}

int Map::addAction(int x, int y, int what, int t)
{
	int i;

	for (i=0; i<MAX_LLAMAS; i++) {
		if (actions[i].what==-1) {
			actions[i].x=x;
			actions[i].y=y;
			actions[i].what=what;
			actions[i].t=t;
			action_count++;
			return 0;
		}
	}
	return 1;
}

int Map::iterate()
{
	int i, j;

	time++;

	//start new llamas
	if (((speedMax-llamaStartSpeed) <= (time-llamasLastStart)) && llamasToStart) {
		llamasToStart--;
		llamasLastStart=time;

		llamas[llama_count].setAlive();
		llamas[llama_count].setPos(start.x, start.y);

		llama_count++;
	}

	//'perform' actions
	for (j=0,i=0; i<MAX_LLAMAS && j<action_count; i++) {
		if (actions[i].what!=-1) {
			if ((actions[i].t--)<1)
				actions[i].what=-1;
			j++;
		}
	}

	SDL_LockSurface(map);
	if (solid)
		SDL_LockSurface(solid);
	//process all llamas
	for (i=0; i<llama_count; i++) {
		if (llamas[i].isAlive()) { //only process, if he's alive :)
			llamas[i].move(*this);
		}
	}

	if (solid)
		SDL_UnlockSurface(solid);
	SDL_UnlockSurface(map);
	//other things
	return 0;
}

LLama::LLama(int xp, int yp)
{
	isalive=0;
	phase=0;
	
	action=ITEM_WALK;
	param=0;
	
	x=xp;
	y=yp;
	
	vx=1;
	vy=0;
	
	ax=0;
	ay=0;
}

void LLama::setAction(int _action)
{
	action=_action;
	switch (action) {
	case ITEM_WALK:
		param=PARAM_WALK;
		break;
	case ITEM_DIG:
		param=PARAM_DIG;
		break;
	case ITEM_EXPLODE:
		param=PARAM_EXPLODE;
		break;
	case ITEM_STAIRS:
		param=PARAM_STAIRS;
		break;
	case ITEM_DIGGER:
		param=PARAM_DIGGER;
		break;
	case ITEM_BLOCK:
		param=PARAM_BLOCK;
		break;
	case ITEM_CLIMB:
		param=PARAM_CLIMB;
		break;
	case ITEM_WINGS:
		param=PARAM_WINGS;
		break;
	}
}

void LLama::move(Map &map)
{
	doAction(map);
	doPhysics(map);
}

void LLama::doPhysics(Map &map)
{
	int j, k, l, stop;

	ay++;
	
	vy+=ay;
	
	//slow down if llama has wings
	if (getAction()==ITEM_WINGS) {
		if (vy>50)
			vy=50;
	}
	//EQ2: y+=vy/25;
	stop=0;

	for (j=0; j<=myabs(vy)/25; j++) {
		//if llama get's behind the map border, kill her
		if (y+LLAMA_SIZE>=map.getMapImage()->h || y<0) {
			die();
			break;
		}
		for (k=0; k<LLAMA_SIZE; k++) {
			if (map.mapIsGround(x+k, y+LLAMA_SIZE)) { //TODO?: lepsia detekcia kolizii
				if (vy>350) { //llama has fallen too fast
					die();
					//make a rise
					for (l=-4; l<LLAMA_SIZE+4; l++) {
						map.mapSetGround(x+l, y+LLAMA_SIZE, 0);
						if (l>-4 && l<LLAMA_SIZE+3)
							map.mapSetGround(x+l, y+LLAMA_SIZE-1, 0);
						if (l>-3 && l<LLAMA_SIZE+2)
							map.mapSetGround(x+l, y+LLAMA_SIZE-2, 0);
					}
				}
				ay=0;
				if (vy>0)
					vy=0;
				stop=1;
				break;
			}
		}
		if (stop)
			break;
		if (vy>0)
			y++;
		else y--;
	}
}

void LLama::doAction(Map &map)
{
	switch (action) {
	case ITEM_WALK:
		doWalk(map);
		break;
	case ITEM_DIG:
		doDig(map);
		break;
	case ITEM_EXPLODE:
		doExplode(map);
		break;
	case ITEM_STAIRS:
		doStairs(map);
		break;
	case ITEM_DIGGER:
		doDigger(map);
		break;
	case ITEM_BLOCK:
		doBlock(map);
		break;
	case ITEM_CLIMB:
		doClimb(map);
		break;
	case ITEM_WINGS:
		doWings(map);
		break;
	default:
		printf("ERROR: No action assigned to %d!", action);
		break;
	}
}

int LLama::doWalk(Map &map)
{
	int f, j, k, z;
	int rv=0;

	f=0;
	for (k=0; k<LLAMA_SIZE; k++) {
		if (map.mapIsGround(x+k, y+LLAMA_SIZE)) 
			f=1;
	}

	//x+=vx;
	for (j=0; j<myabs(vx); j++) {
		if (x+LLAMA_SIZE>=map.getMapImage()->w || x<0) {
			//llama got out of map => she's dead
			die();
			break;
		}
		for (k=0; k<LLAMA_SIZE; k++) {
			if (vx>0) z=x+LLAMA_SIZE+vx;
			else z=x-1+vx;

			if (INRECT(z, y+k, map.getExit().x, map.getExit().y, map.getExit().w, map.getExit().h)) {
				//llama got home
				die(1);
				map.oneLlamaHome();
				break;
			}

			if (map.mapIsGround(z, y+k) || map.mapIsBlocked(z, y+k)) { //TODO?: better detection of collisions???
				//llama has to turn around...
				if (k<LLAMA_SIZE-2) {
					ax=0;
					rv=1;
					if (f) {
						vx=-vx;
						rv=2;
					}
					goto stop2;
				} else y-=(LLAMA_SIZE-k);
			}
		}
		x+=sign(vx); //do steny sa nedostane, lebo v takom pripade sa uz skocilo na stop2...
	}
stop2:

	return rv;
}

int LLama::doDig(Map &map)
{
	int k, f;

	//check whether we still have to dig (is there some ground?)
	f=0;
	for (k=0; k<LLAMA_SIZE; k++) {
		if (map.mapIsGround(x+k, y+LLAMA_SIZE))
			f=1;
	}

	if (f) { //if yes, dig
		for (k=-4; k<LLAMA_SIZE+4; k++) {
			if (map.mapSetGround(x+k, y+LLAMA_SIZE))
				break;
		}
		y++;
		param--;
	}

	if (param%10 == 1)
		map.addAction(x+27/2, y+27/2, ACTION_DUST, 10);

	if (!f || (param<=0)) {
		setAction(ITEM_WALK);
		return 1;
	}

	return 0;
}

int LLama::doExplode(Map &map)
{
	int i, j;

	param--;
	if (param<=0) { //explode if param<=0
		die();
		//make crater => circle with radius EXPLODE_RADIUS
		for (i=-EXPLODE_RADIUS; i<EXPLODE_RADIUS; i++) {
			for (j=-EXPLODE_RADIUS; j<EXPLODE_RADIUS; j++) {
				if (i*i+j*j < EXPLODE_RADIUS*EXPLODE_RADIUS)
					map.mapSetGround(x+i, y+j);
			}
		}
		map.addAction(x, y, ACTION_EXPLOSION, 8);
	} else doWalk(map); //otherwise continue walking

	return 0;
}

int LLama::doStairs(Map &map)
{
	int i, j, offset;

	if (vx>0)
		offset=LLAMA_SIZE;
	else offset=0;

	if (param>0) {
		if (!(param%5)) { //make a stair once a 5 'steps'
			map.addAction(x, y, ACTION_STAIR, 1);
			for (i=-3; i<4; i++) {
				for (j=0; j<3; j++) {
					if (!((i==-3&&j==0) || (i==3&&j==0) || (i==-3&&j==2) || (i==3&&j==2))) //rounded edges :)
						map.mapSetGround(x+sign(vx)*i+offset, y+LLAMA_SIZE+j-1, 0);
				}
			}
		} else { //walk
			if (doWalk(map)) {
				setAction(ITEM_WALK);
				return 1;
			}
		}
		param--;
	} else { //we're out of bricks >:-D
		setAction(ITEM_WALK);
		return 1;
	}

	return 0;
}

int LLama::doDigger(Map &map)
{
	int k, l, f;

	f=0;
	for (k=0; k<LLAMA_SIZE; k++) {
		if (map.mapIsGround(x+k, y+LLAMA_SIZE))
			f=1;
		if (vx>0) {
			if (map.mapIsGround(x+LLAMA_SIZE+1, y+k))
				f=1;
		} else {
			if (map.mapIsGround(x-1, y+k))
				f=1;
		}
	}

	if (f) {
		if (param==PARAM_DIGGER) { //the first time, clean everything a around the llama
			for (k=-2; k<LLAMA_SIZE+2; k++)
				for (l=-2; l<LLAMA_SIZE+2; l++)
					map.mapSetGround(x+k, y+l);
		}

		for (k=-1; k<LLAMA_SIZE+1; k++) {
			map.mapSetGround(x+k, y+LLAMA_SIZE);
			if (vx>0) {
				if (map.mapSetGround(x+LLAMA_SIZE+1, y+k)) {
					param=1;
					break;
				}
			} else {
				if (map.mapSetGround(x-1, y+k)) {
					param=1;
					break;
				}
			}
		}

		y++;
		if (vx>0)
			x++;
		else x--;
		param--;
	}

	if (param%10 == 1)
		map.addAction(x+27/2, y+27/2, ACTION_DUST, 10);

	if (!f || (param<=0)) {
		setAction(ITEM_WALK);
		return 1;
	}

	return 0;
}

int LLama::doBlock(Map &map)
{
	//Do shit :)
	return 0;
}

int LLama::doClimb(Map &map)
{
	int k, f=0;
	
	//search for wall in the direction of walk...
	for (k=0; k<LLAMA_SIZE; k++) {
		if (vx>0) {
			if (map.mapIsGround(x+LLAMA_SIZE+1, y+k))
				f=1;
		} else {
			if (map.mapIsGround(x-1, y+k))
				f=1;
		}
	}

	if (f) { //if there is some then climb:
		f=0;
		for (k=0; k<LLAMA_SIZE; k++) {
			if (map.mapIsGround(x+k, y-1))
				f=1;
		}
		if (f==1) {
			setAction(ITEM_WALK);
			return 1;
		}
		//printf("Found a wall!! :)");
		y-=3;
		vy=0;
		ay=0;
		param--;
		if (param==0) {
			setAction(ITEM_WALK);
			return 1;
		}
	} else //else walk
		return doWalk(map);
	return 0;
}

int LLama::doWings(Map &map)
{
	//works the same as walking...
	return doWalk(map);
}

void LLama::die(int die)
{
	if (die==0)
		my_playSound(SOUND_KILLED, x, y);
	else
		my_playSound(SOUND_YIPEE, x, y);
	setAlive(0);
}