/*
    LLamy
    Copyright (C) 2004 Andrej Krutak

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _MAP_H
#define _MAP_H

#include <sdl.h>
#include <sdl_image.h>

int inline min(int a, int b)
{
	return a<b ? a : b;
}

int inline max(int a, int b)
{
	return a<b ? b : a;
}

int inline INRECT(int x, int y, int x1, int y1, int w, int h)
{
	if (x>=x1 && y>=y1 && x<=(x1+w) && y<=(y1+h))
		return 1;
	else return 0;
}

enum {
	MAX_LLAMAS=500,
	LLAMA_SIZE=27,
	EXPLODE_RADIUS=LLAMA_SIZE*3,

	PARAM_WALK=0,
	PARAM_DIG=100,
	PARAM_EXPLODE=100,
	PARAM_STAIRS=120,
	PARAM_DIGGER=100,
	PARAM_BLOCK=0,
	PARAM_CLIMB=150,
	PARAM_WINGS=0,

	ITEM_WALK=0,
	ITEM_DIG,
	ITEM_EXPLODE,
	ITEM_STAIRS,
	ITEM_DIGGER,
	ITEM_BLOCK,
	ITEM_CLIMB,
	ITEM_WINGS,
	ITEM_ATOMBOMB,
	MAP_NO_ITEMS,

	ACTION_EXPLOSION=0,
	ACTION_DUST,
	ACTION_STAIR,
	MAP_NO_ACTIONS,
};

typedef struct {
	int x, y;
	int what;
	int t;
} map_action;

class Map;

class LLama {
private:
	int phase; //movement phase
	int isalive; //if the lama is dead, it's 0

	int x, y; //position
	int vx, vy; //speed
	int ax, ay; //acceleration
	
	int action; //the current job
	int param; //parameter of the 'job'
public:
	LLama(int xp=0, int yp=0);
	~LLama() {}

	int getPhase() {return phase;}
	int getParam() {return param;}
	int isAlive() {return isalive;}
	int getX() {return x;}
	int getY() {return y;}
	int getDirection() {return vx>0?1:-1;}
	int getAction() {return action;}
	
	void setPos(int xp, int yp) {x=xp; y=yp;}
	void setAlive(int alive=1) {isalive=alive;}
	void setAction(int action); //sets 'action' as the current job, and also the appropiate 'param'

	void move(Map &map); //does one lifestep (doPhysics(), doAction()...)
	void doPhysics(Map &map); //applies physics
	void doAction(Map &map); //does the current job

	void die(int how=0); //kills the llama; the how!=0 means, that the llama went home :)

	//these functions do, what you would expect...
	//they return 1 if they change the current action of the llama...
	int doWalk(Map &map);
	int doDig(Map &map);
	int doExplode(Map &map);
	int doStairs(Map &map);
	int doDigger(Map &map);
	int doBlock(Map &map);
	int doClimb(Map &map);
	int doWings(Map &map);
};

class Map {
friend class LLama;
private:
	//some internal things, their meaning is obvious
	SDL_Surface *map, *bk, *solid;
	SDL_Color bkColor, grColor;
	SDL_Rect start, exit;
	int loaded; //level loaded successfully

	int llamyStart, llamyEnd, llamyHome;
	int llamaStartSpeed, speedMin, speedMax, speedStart; //time per start
	int llamasToStart, llamasLastStart;
	
	//internals
	int items[MAP_NO_ITEMS];
	LLama llamas[MAX_LLAMAS];
	int llama_count;

	//multimedia effects (like explosions...) :-)
	map_action actions[MAX_LLAMAS];
	int action_count;

	int time;
public:
	Map(char *path);
	~Map();

	int isLoaded() {return loaded;}

	SDL_Surface *getMapImage() {return map;}
	SDL_Surface *getBkImage() {return bk;}
	SDL_Rect getStart() {return start;}
	SDL_Rect getExit() {return exit;}

	int getLlamaCount() {return llama_count;}
	int getLlamaMax() {return llamyStart;}
	int getLlamaHome()  {return llamyHome;}
	int getLlamaHomeMax()  {return llamyEnd;}
	int getTime() {return time;}
	int noMoreLLamas(int alive) {return (llamasToStart==0) && (alive==0);}

	int getSpeedMin() {return speedMin;}
	int getSpeedCur() {return llamaStartSpeed;}
	int getSpeedMax() {return speedMax;}


	void setSpeed(int spd) {llamaStartSpeed=min(max(getSpeedMin(), spd), getSpeedMax());} //sets the current llama starting speed (if it's OK)
	int addAction(int x, int y, int what, int t); //adds an action 'what', on place 'x', 'y' on the map, with lifetime 't'
	int getActionCount() {return action_count;}
	map_action const* getAction(int no) {return &actions[no];}

	LLama* getLlama(int i) {if (i>=llama_count || i<0) throw "getLLama_i_out_of_range"; return &llamas[i];} //returns pointer to llama no. 'i'
	int getItemCount(int i) {if (i>=0 && i<MAP_NO_ITEMS) return items[i]; else return -1;} //returns the count of item 'i'
	void decItemCount(int i) {if (i>=0 && i<MAP_NO_ITEMS) items[i]--;} //decreases the count of item 'i'

	void oneLlamaHome() {llamyHome++;} //called when 1 llama gets home

	int inline mapIsGround(int x, int y); //returns 1 if there's ground at 'x', 'y' in the map; map should be SDL_Locked
	int inline mapSetGround(int x, int y, int erase=1); //if map isn't solid at 'x', 'y' => clears map at 'x', 'y' if erase=='1', fills otherwise; map and solid should be SDL_Locked
	int mapIsBlocked(int x, int y); //returns 1 if there is a 'blocking' llama on the place 'x', 'y' in the map

	int iterate(); //performs 1 step in the time (simulates all what's needed ;-)
};

#endif //_LLAMY_H
