/*
    LLamy
    Copyright (C) 2004 Andrej Krutak

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <sdl.h>
#include <sdl_image.h>
#include <sge.h>
#include <time.h>

#include "map.h"
#include "func.h"

//some externals
extern SDL_Surface *screen;
extern int flip;
extern int sw;
extern int sh;

//screen position and current map...
static int sx, sy;
static Map *map;

enum {
	HIG_BOX=1,
};

typedef struct {
	char *path;
	SDL_Surface **to;
	Uint8 transparentr, transparentg, transparentb;
} images_to_load;

//main HUGE play-game loop :-)
int playLevel(char *path, char *name)
{
	int posx, posy, end;
	SDL_Event event;
	int x, y;
	int bx, by;
	int mx, my;
	int i, j, k;
	SDL_Surface *items, *font, *font2, *logo, *explode, *timg, *dust;
	SDL_Rect r;
	char buf[1000];
	int curitem=-1;
	int tcks=1, oldtcks=0, frms=20;
	int debug=1;
	int t, llalive;
	int spdupd=0;
	LLama *l;
	int finishedq=0;
	Mix_Music *mus=0;
	int f, ff;
	char keypressed[SDLK_LAST]; memset(keypressed, 0, sizeof(keypressed));
	Uint32 timer;
	const int FPSLIMITER_MINTIME=1000/25;

	SDL_Surface *llama_motion[MAP_NO_ITEMS];
	int llama_motion_steps[MAP_NO_ITEMS];
	SDL_Surface *llama_motion2[MAP_NO_ITEMS];

	//Load all images
	images_to_load itl[] = {
		{"img/misc.png", &items, 255, 0, 255},
		{"img/font.png", &font, 255, 0, 255},
		{"img/font_big.png", &font2, 255, 0, 255},
		{"img/logo.png", &logo, 255, 0, 255},
		{"img/explode.png", &explode, 255, 255, 255},
		{"img/dust.png", &dust, 255, 0, 255},

		{"img/llama_motion0.png", &llama_motion[0], 255, 0, 255},
		{"img/llama_motion1.png", &llama_motion[1], 255, 0, 255},
		{"img/llama_motion2.png", &llama_motion[2], 255, 0, 255},
		{"img/llama_motion3.png", &llama_motion[3], 255, 0, 255},
		{"img/llama_motion4.png", &llama_motion[4], 255, 0, 255},
		{"img/llama_motion5.png", &llama_motion[5], 255, 0, 255},
		{"img/llama_motion6.png", &llama_motion[6], 255, 0, 255},
		{"img/llama_motion7.png", &llama_motion[7], 255, 0, 255},
		{"img/llama_motion8.png", &llama_motion[8], 255, 0, 255},
	};

	printf("Play level: %s\n", path);
	
	for (i=0; i<sizeof(itl)/sizeof(images_to_load); i++) {
		*itl[i].to=IMG_Load(itl[i].path);
		if (*itl[i].to==0) {
			printf("! %s\n", itl[i].path);
			for (j=0; j<i; j++)
				SDL_FreeSurface(*itl[j].to);
			return 1;
		}
		SDL_SetColorKey(*itl[i].to, SDL_SRCCOLORKEY, SDL_MapRGB((*itl[i].to)->format, itl[i].transparentb, itl[i].transparentg, itl[i].transparentb));
	}

	//make right->left direction images for the llama movement...
	SDL_SetAlpha(logo, SDL_SRCALPHA, 180);

	for (k=0; k<MAP_NO_ITEMS; k++) {
		llama_motion_steps[k]=llama_motion[k]->w/LLAMA_SIZE;
		
		llama_motion2[k]=sge_copy_surface(llama_motion[k]);
		SDL_LockSurface(llama_motion[k]);
		SDL_LockSurface(llama_motion2[k]);
		for (i=0; i<llama_motion[k]->w; i++) {
			for (j=0; j<llama_motion[k]->h; j++) {
				_PutPixel(llama_motion2[k], llama_motion[k]->w-i-1, j, sge_GetPixel(llama_motion[k], i, j));
			}
		}
		SDL_UnlockSurface(llama_motion[k]);
		SDL_UnlockSurface(llama_motion2[k]);
	}

	//load the map
	map=new Map(path);
	if (!map->isLoaded())
		goto term;

	//show 'welcome' screen for the map
	timg=IMG_Load("img/loadlev.jpg");
	drawText(timg, 8, 413, font2, "%s", name);
	showImageFullscreen(timg, 3000);
	SDL_FreeSurface(timg);

	//some initialisation of internals
	end=0;
	curitem=ITEM_WALK;

	mx=map->getMapImage()->w;
	my=map->getMapImage()->h;
	bx=map->getBkImage()->w;
	by=map->getBkImage()->h;
	sx=screen->w;
	sy=screen->h;

	posx=map->getStart().x-sx/2;
	if (posx<0) posx=0;
	if (posx+sx>mx) posx=mx-sx;
	if (mx<sx)
		posx=-(sx-mx)/2;

	posy=map->getStart().y-sy/2;
	if (posy<0) posy=0;
	if (posy+sy>my) posy=my-sy;
	if (my<sy)
		posy=-(sy-my)/2;

	//play music
	f=0;
	ff=0;

	while (f<10 && !ff) {
		ff=rand()%MUSIC_NOITEMS;
		
		sprintf(buf, "sound/%02d.ogg", ff);
		mus=my_playMusicStart(buf);

#ifdef MP3_MUSIC		
		if (!mus) {
			sprintf(buf, "sound/%02d.mp3", ff);
			mus=my_playMusicStart(buf);
		}
#endif

		if (!mus) {
			f++;
			ff=0;
			printf("!%s\n", buf);
		} else ff=1;
	}

	timer=SDL_GetTicks()-FPSLIMITER_MINTIME-1;
	//main loop
	while (!end) {
		//redraw
		int xp, yp;
		if (sx==mx) xp=1;
		else xp=mx-sx;
		if (sy==my) yp=1;
		else yp=my-sy;

		drawImage(screen, map->getBkImage(), posx*(bx-sx)/xp, posy*(by-sy)/yp, sx, sy, 0, 0);
		drawImage(screen, map->getMapImage(), posx, posy, sx, sy, 0, 0);
		
		//draw map items
		r=map->getStart();
		r.w=40;
		r.h=40;
		drawItemAtRect(items, 0, 40, r, posx, posy, sx, sy);
		
		r=map->getExit();
		r.w=40;
		r.h=40;
		drawItemAtRect(items, 40, 40, r, posx, posy, sx, sy);

		llalive=0;
		t=map->getLlamaCount();
		for (i=0; i<t; i++) {
			l=map->getLlama(i);
			if (l->isAlive()) {
				llalive++;
				r.x=l->getX();
				r.y=l->getY();
				r.w=LLAMA_SIZE;
				r.h=LLAMA_SIZE;
				drawItemAtRect((l->getDirection()>0)?llama_motion[l->getAction()]:llama_motion2[l->getAction()], ( (l->getPhase()+map->getTime())%llama_motion_steps[l->getAction()])*LLAMA_SIZE, 0, r, posx, posy, sx, sy);

				if (l->getAction()==ITEM_EXPLODE) {
					int x=l->getX();
					int y=l->getY()-10;

					if (! ((x<posx || y<posy) || (x>posx+sx || y>posy+sy)) )
						drawText(screen, x-posx, y-posy, font, "T-%d", l->getParam());
				}
			}
		}

		for (j=0,i=0; i<MAX_LLAMAS && j<map->getActionCount(); i++) {
			if (map->getAction(i)->what!=-1) {
				switch (map->getAction(i)->what) {
				case ACTION_EXPLOSION:
				case ACTION_DUST:
					{
					int h;
					int w;
					SDL_Surface *img=0;

					if (map->getAction(i)->what==ACTION_EXPLOSION) {
						img=explode;
						if (map->getAction(i)->t==7) {
							//printf("%d [%d:%d]; ", map->getAction(i)->t, map->getAction(i)->x, map->getAction(i)->y);
							my_playSound(SOUND_EXPLODE, map->getAction(i)->x, map->getAction(i)->y);
						}
					} else if (map->getAction(i)->what==ACTION_DUST) {
						img=dust;
						if (map->getAction(i)->t==9)
							my_playSound(SOUND_DIG, map->getAction(i)->x, map->getAction(i)->y);
					}

					h=img->h;
					w=h;
					r.x=map->getAction(i)->x-w/2;
					r.y=map->getAction(i)->y-h/2;
					r.w=w;
					r.h=h;
					drawItemAtRect(img, (img->w/w-map->getAction(i)->t-1)*w, 0, r, posx, posy, sx, sy);
					}
					break;
				case ACTION_STAIR:
					my_playSound(SOUND_STAIR, map->getAction(i)->x, map->getAction(i)->y);
					break;
				}
				//if ((map->getAction(i)->t--)<1)
				//	map->getAction(i)->what=0;
				j++;
			}
		}
		//user interface
		if (curitem!=-1) {
			Uint32 clr;
			SDL_LockSurface(screen);
			if (map->getItemCount(curitem)>0)
				clr=SDL_MapRGB(screen->format, 255, 255, 255);
			else clr=SDL_MapRGB(screen->format, 255, 100, 100);
			if (HIG_BOX) {
				for (i=0; i<40; i++) {
					for (j=0; j<55; j++) {
						_PutPixelAlpha(screen, curitem*40+i, j, clr, 100);
					}
				}
			} else {
				for (i=0; i<40; i++) {
					_PutPixelAlpha(screen, curitem*40+0, 0+i, clr, 255);
					_PutPixelAlpha(screen, curitem*40+i, 0, clr, 255);
					_PutPixelAlpha(screen, curitem*40+i, 40, clr, 255);
					_PutPixelAlpha(screen, curitem*40+40, i, clr, 255);
				}
			}
			SDL_UnlockSurface(screen);
		}
		for (i=0; i<MAP_NO_ITEMS; i++) {
			drawImage(screen, items, i*40, 0, 40, 40, i*40, 0);
			drawText(screen, i*40+15, 40, font, "%d", map->getItemCount(i));
		}

		drawText(screen, 5, 440, font, "Llamas alive: %d/%d; Llamas home: %d/%d; Time: %d\nStart speed: [%d] %d [%d]", llalive, map->getLlamaMax(), map->getLlamaHome(), map->getLlamaHomeMax(), map->getTime(), map->getSpeedMin(), map->getSpeedCur(), map->getSpeedMax());
		drawImage(screen, logo, 0, 0, logo->w, logo->h, screen->w-logo->w-5, screen->h-logo->h-5);

		//debug info
		frms--;
		if (!frms) {
			frms=20;
			oldtcks=tcks;
			tcks=SDL_GetTicks();
		}
		if (debug) {
			drawText(screen, 500, 0, font, "FPS: %d", 20*1000/(tcks-oldtcks));
		}

		//if the user get's the required number of llamas home, he can exit
		if (map->getLlamaHome()>=map->getLlamaHomeMax()) {
			SDL_Rect yes={66, 97, 65, 34};
			SDL_Rect no={155, 97, 65, 34};
			if (!finishedq&&giveQuestion("img/victory.png", yes, no))
				end=3;
			else finishedq=1;
		}

		//if all llamas are dead and/or home, exit the game (check if he won...)
		if (end!=2 && map->noMoreLLamas(llalive)) {
			SDL_Rect yes={122, 97, 55, 32};
			SDL_Rect no=yes;
			giveQuestion("img/done.png", yes, no);
			if (map->getLlamaHome()>=map->getLlamaHomeMax())
				end=3;
			else end=2;
		}
		updateScreen();

		//other things...
		while ( SDL_PollEvent(&event) ) {
			switch (event.type) {
				case SDL_KEYDOWN:
					if (event.key.keysym.sym==SDLK_ESCAPE) { //quit?
						if (map->getLlamaHome()<map->getLlamaHomeMax()) {
							SDL_Rect yes={66, 97, 65, 34};
							SDL_Rect no={155, 97, 65, 34};
							if (giveQuestion("img/quit.png", yes, no))
								end=1;
						} else {
							SDL_Rect yes={66, 97, 65, 34};
							SDL_Rect no={155, 97, 65, 34};
							if (giveQuestion("img/victory.png", yes, no))
								end=3;
						}
					}
					else if (event.key.keysym.sym=='d') //show/hide debug info
						debug=!debug;
					else if (event.key.keysym.sym=='q') //increase launch speed
						spdupd=1;
					else if (event.key.keysym.sym=='a') //increase launch speed
						spdupd=-1;
					for (i='1'; i<='9'; i++) { //change current item
						if (event.key.keysym.sym==i)
							curitem=i-'1';
					}
					keypressed[event.key.keysym.sym]=1;
					break;
				case SDL_KEYUP:
					if (event.key.keysym.sym=='q' || event.key.keysym.sym=='a')
						spdupd=0;
					keypressed[event.key.keysym.sym]=0;
					break;
				case SDL_QUIT:
					end=1;
					break;
				case SDL_MOUSEBUTTONDOWN:
					if (event.button.button==SDL_BUTTON_LEFT && INRECT(event.button.x, event.button.y, 0, 0, MAP_NO_ITEMS*40, 40)) { //selected a item
						curitem=event.button.x/40;
					} else if (event.button.button==SDL_BUTTON_LEFT) {
						if (map->getItemCount(curitem)>0) { //check, if we didn't click llama, if yes, try assignign current action to it
							t=map->getLlamaCount();
							for (i=0; i<t; i++) {
								l=map->getLlama(i);
								if (l->isAlive()) {
									r.x=l->getX();
									r.y=l->getY();
									r.w=LLAMA_SIZE;
									r.h=LLAMA_SIZE;
									if (isInRect(r, posx, posy, sx, sy, event.button.x, event.button.y) && l->getAction()!=curitem) {
										map->decItemCount(curitem);

										if (curitem!=ITEM_ATOMBOMB) {
											l->setAction(curitem);
										} else {
											for (i=0; i<t; i++) {
												l=map->getLlama(i);
												if (l->isAlive())
													l->setAction(ITEM_EXPLODE);
											}
										}
										break;
									}
								}
							}
						}
					}
					break;
				case SDL_MOUSEMOTION:
					if (event.motion.state&SDL_BUTTON(SDL_BUTTON_RIGHT)) { //scroll the screen
						x=event.motion.xrel;
						y=event.motion.yrel;

						x=-x;
						y=-y;
						if (mx>=sx) {
							if (x>0)
								posx=min(posx+x, mx-sx);
							if (x<0)
								posx=max(posx+x, 0);
						}
						
						if (my>=sy) {
							if (y>0)
								posy=min(posy+y, my-sy);
							if (y<0)
								posy=max(posy+y, 0);
						}
					}
					break;
				default:
					break;
			}
		}
		
		if (keypressed[SDLK_UP] || keypressed[SDLK_DOWN]) {
			if (keypressed[SDLK_UP])
				y=-10;
			else y=10;
			if (my>=sy) {
				if (y>0)
					posy=min(posy+y, my-sy);
				if (y<0)
					posy=max(posy+y, 0);
			}
		}
		
		if (keypressed[SDLK_LEFT] || keypressed[SDLK_RIGHT]) {
			if (keypressed[SDLK_LEFT])
				x=-10;
			else x=10;
			if (mx>=sx) {
				if (x>0)
					posx=min(posx+x, mx-sx);
				if (x<0)
					posx=max(posx+x, 0);
			}
		}

		map->setSpeed(map->getSpeedCur()+spdupd);

		if ((SDL_GetTicks()-timer)>FPSLIMITER_MINTIME) {
			timer=SDL_GetTicks();
			if (map->iterate()) //perform 1 step in the world (the time is quantized, didn't you know? :-)
				end=1;
		}
	}

term:
	//cleanup
	if (mus)
		my_playMusicStop(mus);

	delete map;

	for (j=0; j<sizeof(itl)/sizeof(images_to_load); j++)
		SDL_FreeSurface(*itl[j].to);

	return -end;
}

//informations about the current situation in the game ;-)
int screenXPos()
{
	return sx;
}

int screenYPos()
{
	return sy;
}

int mapXSize()
{
	return map->getMapImage()->w;
}

int mapYSize()
{
	return map->getMapImage()->h;
}
